@@ -1,7 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*************************************************************************
  *
- * Copyright (c) 2015-2016 Kohei Yoshida
+ * Copyright (c) 2015-2020 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -35,19 +35,20 @@
 #include <string>
 #include <deque>
 #include <map>
+#include <memory>
 
 namespace mdds {
 
 namespace trie {
 
 /**
- * String trait that uses std::string as the key type.  This trait can be
- * used with mdds::trie_map or mdds::packed_trie_map.
+ * Template for a key type implemented using a typical STL container type.
  */
-struct std_string_trait
+template<typename ContainerT>
+struct std_container_trait
 {
     /** type used to store a key value. */
-    typedef std::string key_type;
+    using key_type = ContainerT;
 
     /**
      * type used to build an intermediate key value, from which a final key
@@ -55,14 +56,14 @@ struct std_string_trait
      * content is contiguous in memory.  Its elements must be of
      * key_unit_type.
      */
-    typedef std::string key_buffer_type;
+    using key_buffer_type = key_type;
 
     /**
      * type that represents a single character inside a key or a key buffer
      * object.  A key object is expected to store a series of elements of this
      * type.
      */
-    typedef char key_unit_type;
+    using key_unit_type = typename key_type::value_type;
 
     /**
      * Function called to create and initialize a buffer object from a given
@@ -138,7 +139,72 @@ struct std_string_trait
     }
 };
 
-}
+using std_string_trait = std_container_trait<std::string>;
+
+/** Serializer for numeric data types. */
+template<typename T>
+struct numeric_value_serializer
+{
+    static constexpr bool variable_size = false;
+
+    static constexpr size_t value_size = sizeof(T);
+
+    static void write(std::ostream& os, const T& v);
+
+    static void read(std::istream& is, size_t n, T& v);
+};
+
+/** Serializer for variable-size data types. */
+template<typename T>
+struct variable_value_serializer
+{
+    static constexpr bool variable_size = true;
+
+    static void write(std::ostream& os, const T& v);
+
+    static void read(std::istream& is, size_t n, T& v);
+};
+
+/**
+ * Serializer for standard sequence container whose value type is of
+ * numeric value type.
+ */
+template<typename T>
+struct numeric_sequence_value_serializer
+{
+    using element_serializer = numeric_value_serializer<typename T::value_type>;
+
+    static constexpr bool variable_size = true;
+
+    static void write(std::ostream& os, const T& v);
+
+    static void read(std::istream& is, size_t n, T& v);
+};
+
+/**
+ * Default value serializer for mdds::packed_trie_map's load_state and
+ * save_state methods.  The primary template is used for numeric value
+ * types, and template specializations exist for std::string, as
+ * well as sequence containers, such as std::vector, whose elements are of
+ * numeric types.
+ */
+template<typename T, typename U = void>
+struct value_serializer : numeric_value_serializer<T>
+{
+};
+
+template<typename T>
+struct value_serializer<T, typename std::enable_if<has_value_type<T>::value>::type>
+    : numeric_sequence_value_serializer<T>
+{
+};
+
+template<>
+struct value_serializer<std::string> : variable_value_serializer<std::string>
+{
+};
+
+} // namespace trie
 
 template<typename _KeyTrait, typename _ValueT>
 class packed_trie_map;
@@ -153,24 +219,29 @@ template<typename _KeyTrait, typename _ValueT>
 class trie_map
 {
     friend class packed_trie_map<_KeyTrait, _ValueT>;
-    friend class trie::detail::iterator_base<trie_map>;
+    friend class trie::detail::iterator_base<trie_map, true>;
+    friend class trie::detail::iterator_base<trie_map, false>;
+    friend class trie::detail::const_iterator<trie_map>;
+    friend class trie::detail::iterator<trie_map>;
     friend class trie::detail::search_results<trie_map>;
+    friend trie::detail::get_node_stack_type<trie_map, std::true_type>;
+    friend trie::detail::get_node_stack_type<trie_map, std::false_type>;
 
 public:
     typedef packed_trie_map<_KeyTrait, _ValueT> packed_type;
     typedef _KeyTrait key_trait_type;
     typedef typename key_trait_type::key_type key_type;
     typedef typename key_trait_type::key_buffer_type key_buffer_type;
-    typedef typename key_trait_type::key_unit_type   key_unit_type;
+    typedef typename key_trait_type::key_unit_type key_unit_type;
     typedef _ValueT value_type;
     typedef size_t size_type;
     typedef std::pair<key_type, value_type> key_value_type;
 
-    typedef trie::detail::iterator_base<trie_map> const_iterator;
+    using const_iterator = trie::detail::const_iterator<trie_map>;
+    using iterator = trie::detail::iterator<trie_map>;
     typedef trie::detail::search_results<trie_map> search_results;
 
 private:
-
     struct trie_node
     {
         typedef std::map<key_unit_type, trie_node> children_type;
@@ -179,42 +250,64 @@ private:
         value_type value;
         bool has_value;
 
-        trie_node() : has_value(false) {}
+        trie_node();
+        trie_node(const trie_node& other);
+        trie_node(trie_node&& other);
+
+        void swap(trie_node& other);
     };
 
+    template<bool _IsConst>
     struct stack_item
     {
-        typedef typename trie_node::children_type::const_iterator child_pos_type;
-        const trie_node* node;
+        using _is_const = bool_constant<_IsConst>;
+
+        using child_pos_type = typename get_iterator_type<typename trie_node::children_type, _is_const>::type;
+
+        using trie_node_type = typename const_or_not<trie_node, _is_const>::type;
+
+        trie_node_type* node;
         child_pos_type child_pos;
 
-        stack_item(const trie_node* _node, const child_pos_type& _child_pos) :
-            node(_node), child_pos(_child_pos) {}
+        stack_item(trie_node_type* _node, const child_pos_type& _child_pos) : node(_node), child_pos(_child_pos)
+        {}
 
-        bool operator== (const stack_item& r) const
+        bool operator==(const stack_item& r) const
         {
             return node == r.node && child_pos == r.child_pos;
         }
 
-        bool operator!= (const stack_item& r) const
+        bool operator!=(const stack_item& r) const
         {
-            return !operator== (r);
+            return !operator==(r);
         }
     };
 
-    typedef std::vector<stack_item> node_stack_type;
+    using const_node_stack_type = std::vector<stack_item<true>>;
+    using node_stack_type = std::vector<stack_item<false>>;
 
 public:
-
     /**
      * Default constructor.
      */
     trie_map();
 
+    trie_map(const trie_map& other);
+
+    trie_map(trie_map&& other);
+
     const_iterator begin() const;
 
+    iterator begin();
+
     const_iterator end() const;
 
+    iterator end();
+
+    trie_map& operator=(trie_map other);
+
+    void swap(trie_map& other);
+
     /**
      * Insert a new key-value pair.
      *
@@ -249,8 +342,8 @@ public:
      *
      * @param key key to match.
      *
-     * @return iterator that references a value associated with the key, or
-     *         the end position in case the key is not found.
+     * @return immutable iterator that references a value associated with the
+     *         key, or the end position in case the key is not found.
      */
     const_iterator find(const key_type& key) const;
 
@@ -261,11 +354,33 @@ public:
      *              match.
      * @param len length of the matching key value.
      *
-     * @return iterator that references a value associated with the key, or
-     *         the end position in case the key is not found.
+     * @return immutable iterator that references a value associated with the
+     *         key, or the end position in case the key is not found.
      */
     const_iterator find(const key_unit_type* input, size_type len) const;
 
+    /**
+     * Find a value associated with a specified key.
+     *
+     * @param key key to match.
+     *
+     * @return mutable iterator that references a value associated with the
+     *         key, or the end position in case the key is not found.
+     */
+    iterator find(const key_type& key);
+
+    /**
+     * Find a value associated with a specified key.
+     *
+     * @param input pointer to an array whose value represents the key to
+     *              match.
+     * @param len length of the matching key value.
+     *
+     * @return mutable iterator that references a value associated with the
+     *         key, or the end position in case the key is not found.
+     */
+    iterator find(const key_unit_type* input, size_type len);
+
     /**
      * Retrieve all key-value pairs whose keys start with specified prefix.
      * You can also retrieve all key-value pairs by passing a null prefix and
@@ -273,7 +388,8 @@ public:
      *
      * @param prefix prefix to match.
      *
-     * @return results object containing all matching key-value pairs.
+     * @return results object that contains all matching key-value pairs. The
+     *         results are sorted by the key in ascending order.
      */
     search_results prefix_search(const key_type& prefix) const;
 
@@ -298,6 +414,8 @@ public:
      */
     size_type size() const;
 
+    bool empty() const noexcept;
+
     /**
      * Empty the container.
      */
@@ -319,9 +437,13 @@ private:
     const trie_node* find_prefix_node(
         const trie_node& node, const key_unit_type* prefix, const key_unit_type* prefix_end) const;
 
+    template<bool _IsConst>
     void find_prefix_node_with_stack(
-        node_stack_type& node_stack,
-        const trie_node& node, const key_unit_type* prefix, const key_unit_type* prefix_end) const;
+        std::vector<stack_item<_IsConst>>& node_stack, const_t<trie_node, _IsConst>& node, const key_unit_type* prefix,
+        const key_unit_type* prefix_end) const;
+
+    template<bool _IsConst>
+    key_buffer_type build_key_buffer_from_node_stack(const std::vector<stack_item<_IsConst>>& node_stack) const;
 
     void count_values(size_type& n, const trie_node& node) const;
 
@@ -349,7 +471,7 @@ public:
     typedef _KeyTrait key_trait_type;
     typedef typename key_trait_type::key_type key_type;
     typedef typename key_trait_type::key_buffer_type key_buffer_type;
-    typedef typename key_trait_type::key_unit_type   key_unit_type;
+    typedef typename key_trait_type::key_unit_type key_unit_type;
     typedef _ValueT value_type;
     typedef size_t size_type;
     typedef std::pair<key_type, value_type> key_value_type;
@@ -366,8 +488,9 @@ public:
         size_type keylen;
         value_type value;
 
-        entry(const key_unit_type* _key, size_type _keylen, value_type _value) :
-            key(_key), keylen(_keylen), value(_value) {}
+        entry(const key_unit_type* _key, size_type _keylen, value_type _value)
+            : key(_key), keylen(_keylen), value(_value)
+        {}
     };
 
 private:
@@ -378,7 +501,8 @@ private:
 
         std::deque<trie_node*> children;
 
-        trie_node(key_unit_type _key) : key(_key), value(nullptr) {}
+        trie_node(key_unit_type _key) : key(_key), value(nullptr)
+        {}
     };
 
     struct stack_item
@@ -387,15 +511,16 @@ private:
         const uintptr_t* child_pos;
         const uintptr_t* child_end;
 
-        stack_item(const uintptr_t* _node_pos, const uintptr_t* _child_pos, const uintptr_t* _child_end) :
-            node_pos(_node_pos), child_pos(_child_pos), child_end(_child_end) {}
+        stack_item(const uintptr_t* _node_pos, const uintptr_t* _child_pos, const uintptr_t* _child_end)
+            : node_pos(_node_pos), child_pos(_child_pos), child_end(_child_end)
+        {}
 
-        bool operator== (const stack_item& other) const
+        bool operator==(const stack_item& other) const
         {
             return node_pos == other.node_pos && child_pos == other.child_pos;
         }
 
-        bool operator!= (const stack_item& other) const
+        bool operator!=(const stack_item& other) const
         {
             return !operator==(other);
         }
@@ -420,11 +545,7 @@ private:
     typedef std::vector<std::tuple<size_t, key_unit_type>> child_offsets_type;
 
 public:
-
-    /**
-     * Not implemented.
-     */
-    packed_trie_map() = delete;
+    packed_trie_map();
 
     /**
      * Constructor that initializes the content from a static list of
@@ -433,8 +554,6 @@ public:
      *
      * @param entries pointer to the array of key-value entries.
      * @param entry_size size of the key-value entry array.
-     * @param null_value null value to return when the find method fails to
-     *                   find a matching entry.
      */
     packed_trie_map(const entry* entries, size_type entry_size);
 
@@ -446,10 +565,24 @@ public:
      */
     packed_trie_map(const trie_map<key_trait_type, value_type>& other);
 
+    packed_trie_map(const packed_trie_map& other);
+
+    packed_trie_map(packed_trie_map&& other);
+
+    packed_trie_map& operator=(packed_trie_map other);
+
+    bool operator==(const packed_trie_map& other) const;
+
+    bool operator!=(const packed_trie_map& other) const;
+
     const_iterator begin() const;
 
     const_iterator end() const;
 
+    const_iterator cbegin() const;
+
+    const_iterator cend() const;
+
     /**
      * Find a value associated with a specified key.
      *
@@ -502,14 +635,41 @@ public:
      *
      * @return the number of entries in the map.
      */
-    size_type size() const;
+    size_type size() const noexcept;
+
+    bool empty() const noexcept;
+
+    void swap(packed_trie_map& other);
+
+    /**
+     * Save the state of the instance of this class to an external buffer.
+     *
+     * @param os output stream to write the state to.
+     */
+    template<typename _Func = trie::value_serializer<value_type>>
+    void save_state(std::ostream& os) const;
+
+    /**
+     * Restore the state of the instance of this class from an external
+     * buffer.
+     *
+     * @param is input stream to load the state from.
+     */
+    template<typename _Func = trie::value_serializer<value_type>>
+    void load_state(std::istream& is);
+
+    /**
+     * Dump the structure of the trie content for debugging.  You must define
+     * <code>MDDS_TRIE_MAP_DEBUG</code> in order for this method to generate
+     * output.
+     */
+    void dump_structure() const;
 
 private:
     node_stack_type get_root_stack() const;
 
     void traverse_range(
-        trie_node& root, node_pool_type& node_pool, const entry* start, const entry* end,
-        size_type pos);
+        trie_node& root, node_pool_type& node_pool, const entry* start, const entry* end, size_type pos);
 
     size_type compact_node(const trie_node& node);
     size_type compact_node(const typename trie_map<_KeyTrait, _ValueT>::trie_node& node);
@@ -523,23 +683,27 @@ private:
         const uintptr_t* p, const key_unit_type* prefix, const key_unit_type* prefix_end) const;
 
     void find_prefix_node_with_stack(
-        node_stack_type& node_stack,
-        const uintptr_t* p, const key_unit_type* prefix, const key_unit_type* prefix_end) const;
+        node_stack_type& node_stack, const uintptr_t* p, const key_unit_type* prefix,
+        const key_unit_type* prefix_end) const;
+
+    template<typename _Handler>
+    void traverse_tree(_Handler hdl) const;
+
+    template<typename _Handler>
+    void traverse_buffer(_Handler hdl) const;
 
 #ifdef MDDS_TRIE_MAP_DEBUG
     void dump_node(key_buffer_type& buffer, const trie_node& node) const;
     void dump_trie(const trie_node& root) const;
-    void dump_packed_trie(const std::vector<uintptr_t>& packed) const;
+    void dump_packed_trie() const;
 #endif
 
 private:
-    size_type m_entry_size;
-
     value_store_type m_value_store;
     packed_type m_packed;
 };
 
-}
+} // namespace mdds
 
 #include "trie_map_def.inl"
 
